# rpt32.rb ver 1.0 (encoding: Windows-31J)  2018/05/15
  # Copyright (C) T. Yoshiizumi, 2018 All rights reserved.
require "nkf"
require "win32ole"
#require "win32/sound"
if defined?(Win32)
    include Win32
end

def check_redirect(filename)
  flag = true
  ff = File.open(filename, "rb")
  count = 0
  while count < 2
    line = ff.gets
    break  unless line
    if line =~ /\(no +redirect\)/i then
      flag = false
      break
    end
    count += 1
  end
  ff.close
  flag
end

def prog_paths()
    ruby_path = ENV["RUBY_PATH"]
    unless ruby_path
        ruby_path = "ruby.exe"
    end
    python_path = ENV["PYTHON_PATH"]
    unless python_path
        python_path = "python.exe"
    end
    rcmd_path = ENV["RCMD_PATH"]
    unless rcmd_path
        rcmd_path = "Rcmd.exe"
    end
    return ruby_path, python_path, rcmd_path
end

def get_rver(rptd)
    rver = ""
    rver_n = ""
    wldc = rptd + "R\\*"
    wldc = wldc.gsub(/\\/, "/")
    Dir.glob(wldc) do |w|
        next  unless test(?d, w)
        next  unless w =~ /\/R\/R-\d+\./
        ver = w.sub(/^.+?\/R\/R-(\d.+)$/, "\\1")
        dname = "R-" + ver
        l = ver.split(/\./)
        ver_n = ""
        for i in 0...l.size
            ver_n = ver_n + sprintf("%03d", l[i].to_i)
        end
        if ver_n > rver_n
            rver = dname
            rver_n = ver_n
        end
    end
    return rver
end

def set_env(rptd, rver, ruby_dir, python_dir)
    r_dir = rptd + "R\\" + rver
    pandoc_dir = rptd + "pandoc"
    tex_dir = rptd + "w32tex"
    if ruby_dir != "" then
        ruby_dir = rptd + ruby_dir + "\\bin"
    else
        ruby_dir = rptd + "ruby\\bin"
    end
    if python_dir != "" then
        python_dir = rptd + python_dir
    else
        python_dir = rptd + "python"
    end
    if test(?d, ruby_dir) then
        ENV["RUBY_PATH"] = ruby_dir + "\\ruby.exe"
    end
    if test(?d, python_dir) then
        ENV["PYTHON_PATH"] = python_dir + "\\python.exe"
    end
    if test(?d, tex_dir) then
        ENV["TEX_PATH"] = tex_dir + "\\bin\\ptex2pdf.exe"
    end
    ENV["RPTD"] = rptd.chop
    ENV["PANDOC_PATH"] = pandoc_dir + "\\pandoc.exe"
    ENV["RCMD_PATH"] = r_dir + "\\bin\\i386\\Rcmd.exe"
    ENV["R_HOME"] = r_dir
    ENV["R_LIBS"] = r_dir + "\\library"
    ENV["R_USER"] = ENV["USERNAME"]
    ENV["RPT_WITH"] = rptd + "with"
    ENV["RPT_TMP"] = ENV["HOMEDRIVE"] +
        ENV["HOMEPATH"] + "\\$$rpt"
    pathOrg = ENV["PATH"]
    if !(pathOrg.include?(r_dir) and pathOrg.include?(pandoc_dir)) then
        add = ""
        if test(?d, ruby_dir) then
            add = add + ruby_dir + ";"
        end
        if test(?d, python_dir) then
            add = add + python_dir + ";" + python_dir + "\\Scripts;"
        end
        if test(?d, tex_dir) then
            add = add + tex_dir + "\\texinst;" + tex_dir + "\\bin;"
        end
        add = add +  r_dir + "\\bin\\i386;" + r_dir + "\\bin;"
        add = add + pandoc_dir + ";"
        add = add + rptd + "d2txt\\command;"
        add = add + rptd + "with;"
        ENV["PATH"] = add + pathOrg
    end
end

def read_cfg(cfg_file)
    ruby_dir = ""
    python_dir = ""
    home_dir = ""
    if test(?f, cfg_file) then
        data = File.read(cfg_file)
        lines = data.split(/\n/)
        lines.each do |line|
            next  if line =~ /^\s*$/
            if !(line =~ /^(ruby|python|home)_dir:\s*(.+)$/)
                next
            end
            m1 = $1
            m2 = $2
            case m1
            when "ruby" then ruby_dir = m2
            when "python" then python_dir = m2
            when "home" then home_dir = m2
            end
        end
    end
    if home_dir == "" or !test(?d, home_dir) then
        home_dir = ENV["HOMEDRIVE"] + ENV["HOMEPATH"]
    end
    if ruby_dir != "" then
        ruby_dir = ruby_dir.sub(/\s+$/, "")
    end
    if python_dir != "" then
        python_dir = python_dir.sub(/\s+$/, "")
    end
    if home_dir != "" then
        home_dir = home_dir.sub(/\s+$/, "")
    end
    return ruby_dir, python_dir, home_dir
end

def get_charset(fname)
    str = ""
    File.open(fname, "rb") {|ff|  str = ff.read(nil)}
    res = NKF.guess(str)
    if res == NKF::UNKNOWN or res == Encoding::WINDOWS_31J
        res = "cp932"
    else
        res = res.to_s
    end
    res
end

def rpt_rfiles()
    res = ''
    rwd = ENV["RPT_WITH"]
    if rwd.class == String then
        wldc = rwd + "\\*.R"
        wldc = wldc.gsub(/\\/, "/")
        flist = Dir.glob(wldc)
        flist.each do |ff|
            next  unless test(?f, ff) 
            ff_cset = get_charset(ff)
            res = res + sprintf("source(\"%s\", encoding=\"%s\")\n",
               ff, ff_cset)
        end
    end
    return res
end

def rmd_file(r_file, t_file, script)
    w = rpt_rfiles()
    text = "library(rmarkdown)\nlibrary(knitr)\n" + w +
        sprintf("infile.rmd <- \"%s\"\nrender(infile.rmd)\n", script)
    File.open(r_file, "w") {|ff|  ff.write(text)}
    text = "\\usepackage{luatexja}\n\\hypersetup{unicode=true}\n"
    File.open(t_file, "w") {|ff|  ff.write(text)}
end

def get_rrb()
    rrb_script = ""
    Dir.glob("!*.rb") do |ff|
        if test(?f, ff) then
            rrb_script = ff
            break
        end
    end
    if rrb_script == "" then
        rwd = ENV["RPT_WITH"]
        if rwd.class == String then
            w = rwd + "\\RmdRuby.rb"
            rrb_script = w  if test(?f, w)
        end
    end
    return rrb_script
end

def main()
    dname = __dir__.gsub(/\//, "\\")
    arg0 = dname + "\\rpt32.rb"
    cfg_file = dname + "\\rpt32.cfg"
    ruby_dir, python_dir, home_dir = read_cfg(cfg_file)

    rptd = ""
    if arg0 =~ /^[A-Z]:\\rpt\\/i then
        rptd = $&
    else
        puts "'rpt32.rb' ̒uꏊsKł."
        puts arg0
        if defined?(Win32)
            Sound.play('SystemExclamation', Sound::ALIAS)
        end
        exit()
    end
    rver = get_rver(rptd)
    if rver == "" then
        puts "v̓\tgR݂܂."
        if defined?(Win32)
            Sound.play('SystemExclamation', Sound::ALIAS)
        end
        exit()
    end
    set_env(rptd, rver, ruby_dir, python_dir)

    argc = ARGV.size
    if argc < 1 then
        if home_dir != "" then
            Dir.chdir(home_dir)
        end
        cmd = "cmd.exe /k \"title=rpt32\""
        WIN32OLE.new("WScript.Shell").run cmd
        exit()
    end

    arg1 = ARGV[0]
    base_dir = File.dirname(ARGV.last)
    if base_dir != "" then
        base_dir = base_dir + "\\"  unless base_dir =~ /\\$/
        Dir.chdir(base_dir)
        dname = base_dir.gsub(/\\/, "\\\\\\\\")
        dname = "^" + dname
        dir_ptn = Regexp.compile(dname, Regexp::IGNORECASE)
    else
        dir_ptn = /^$/
    end
    if argc == 1 then
        ext_ptn = /\.(rb|py|bat|cmd|r|rmd|kmd)(\.txt)?$/i
    else
        ext_ptn = /\.(rb|py|bat|cmd)(\.txt)?$/i
    end

    s_out = "X_OUT.TXT"
    s_err = "X_ERROR.TXT"
    s_log = "X_LOG.TXT"
    [s_out, s_err, s_log].each do |ff|
        File.unlink(ff)  if test(?e, ff)
    end

    r_file = "$$MyTemp.r"
    t_file = "$$luatexja.tex"
    [t_file, r_file].each do |ff|
        File.unlik(ff)  if test(?e, ff)
    end

    script = ""
    ext = ""
    redirect_flag = true
    flist = []
    for i in 0...argc
        fname = ARGV[i]
        if dir_ptn =~ fname then
            fname = fname.sub(dir_ptn, "")
        end
        next  if [s_out, s_err, s_log].include?(fname.upcase)
        if ext_ptn =~ fname and script == "" then
            ext = $&.downcase
            script = fname
            if /\.txt$/ =~ ext then
                ext = ext.sub(/\.txt$/, "")
            end
            redirect_flag = check_redirect(ARGV[i])
            next
        end
        flist.push(fname)
    end
    if script == "" then
        puts "vOt@C̎w肪܂."
        printf("arg#1: %s\n", flist[0])
        if defined?(Win32)
            Sound.play('SystemExclamation', Sound::ALIAS)
        end
        exit()
    else
        flist.unshift(script)
    end

    flist_count = flist.size
    ruby_path, python_path, rcmd_path = prog_paths()
    ext2prog = {'.rb'=>ruby_path, '.py'=>python_path,
        '.r'=>rcmd_path, '.bat'=>'call', '.cmd'=>'call'}
    if ext == ".r" then
        cmd = sprintf("%s BATCH --no-save -q --internet2 \"%s\" %s",
            rcmd_path, script, s_log)
    elsif ext == ".rmd" then
        rmd_file(r_file, t_file, script)
        cmd = sprintf("%s BATCH --no-save -q --internet2 \"%s\" %s",
            rcmd_path, r_file, s_log)
    elsif ext == ".kmd" then
        rrb_script = get_rrb()
        redirect_flag = check_redirect(rrb_script)
        prog = ruby_path
        cmd = prog
        cmd = cmd + sprintf(" \"%s\" \"%s\"", rrb_script, script)
    elsif flist_count == 1 then
        prog = ext2prog[ext]
        cmd = prog
        cmd = cmd + sprintf(" \"%s\"", script)
    else
        prog = ext2prog[ext]
        cmd = prog
        for x in flist
            cmd = cmd + sprintf(" \"%s\"", x)
        end
    end
    if ext != ".r" and redirect_flag then
        cmd = cmd + sprintf(" >%s 2>%s", s_out, s_err)
    end
    printf("\n%s̎s.\n", script)
    system(cmd)
    for ff in [s_out, s_err, s_log]
        File.unlink(ff)  if test(?z, ff)
    end
    for ff in [r_file, t_file]
        File.unlink(ff)  if test(?f, ff)
    end
    Sound.play('SystemAsterisk', Sound::ALIAS)  if defined?(Win32)
end

main()
